@@ -523,6 +523,10 @@ gsm_util_export_activation_environment (GError     **error)
                 return FALSE;
         }
 
+        if (child_environment == NULL) {
+                child_environment = g_listenv ();
+        }
+
         g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{ss}"));
         for (entry_names = g_listenv (); entry_names[i] != NULL; i++) {
                 const char *entry_name = entry_names[i];
@@ -540,6 +544,9 @@ gsm_util_export_activation_environment (GError     **error)
                 if (!g_regex_match (value_regex, entry_value, 0, NULL))
                     continue;
 
+                child_environment = g_environ_setenv (child_environment,
+                                                      entry_name, entry_value,
+                                                      TRUE);
                 g_variant_builder_add (&builder, "{ss}", entry_name, entry_value);
         }
         g_regex_unref (name_regex);
@@ -681,6 +688,39 @@ gsm_util_update_user_environment (const char  *variable,
 
         return environment_updated;
 }
+
+gboolean
+gsm_util_start_systemd_unit (const char  *unit,
+                             const char  *mode,
+                             GError     **error)
+{
+        g_autoptr(GDBusConnection) connection = NULL;
+        g_autoptr(GVariant)        reply = NULL;
+        GError          *bus_error = NULL;
+
+        connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, error);
+
+        if (connection == NULL)
+                return FALSE;
+
+        reply = g_dbus_connection_call_sync (connection,
+                                             "org.freedesktop.systemd1",
+                                             "/org/freedesktop/systemd1",
+                                             "org.freedesktop.systemd1.Manager",
+                                             "StartUnit",
+                                             g_variant_new ("(ss)",
+                                                            unit, mode),
+                                             NULL,
+                                             G_DBUS_CALL_FLAGS_NONE,
+                                             -1, NULL, &bus_error);
+
+        if (bus_error != NULL) {
+                g_propagate_error (error, bus_error);
+                return FALSE;
+        }
+
+        return TRUE;
+}
 #endif
 
 void
